import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _last from "lodash/last";
import _throttle from "lodash/throttle";
import _delay from "lodash/delay";
import _size from "lodash/size";
import _get from "lodash/get";
import React from 'react';
import Lib from 'chat-message-container';
import Sound from 'react-sound';
import * as sound from '../../sounds/sound';
import { getMessages, onScrollToBottom } from '../../../store/actions';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
var MessageContainer = Lib.MessageContainer;

var ChatMessageContainer = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(ChatMessageContainer, _React$Component);

  function ChatMessageContainer(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "scrollToBottom", function () {
      if (_this.refs.chatMessageContainer) {
        _delay(function () {
          _this.refs.chatMessageContainer && _this.refs.chatMessageContainer.scrollToBottom();

          _delay(function () {
            _this.props.actions.onScrollToBottom();
          }, 500);
        }, 500);
      }
    });

    _defineProperty(_assertThisInitialized(_this), "isScrollingOnBottom", function () {
      if (_this.refs.chatMessageContainer) {
        return _this.refs.chatMessageContainer.isAtBottom();
      }

      return false;
    });

    _defineProperty(_assertThisInitialized(_this), "handleSoundStartFinishedPlaying", function (value) {
      _this.setState({
        playSoundForNewMessage: value
      });
    });

    _defineProperty(_assertThisInitialized(_this), "loadMoreMessages", function () {
      var _this$props = _this.props,
          isLoadingMessage = _this$props.isLoadingMessage,
          setting = _this$props.setting;
      if (isLoadingMessage) return;

      _this.throttledLoadMessageHistory(setting.channelId);
    });

    _defineProperty(_assertThisInitialized(_this), "isScrollingOnTop", function () {
      if (_this.refs.chatMessageContainer) {
        var _this$refs$chatMessag = _this.refs.chatMessageContainer.scrollbarRef.getValues(),
            top = _this$refs$chatMessag.top;

        if (top === 0) {
          return true;
        }
      }

      return false;
    });

    _defineProperty(_assertThisInitialized(_this), "scrollToPositionBeforeLoadMore", function () {
      var messageContainerHeight = _this.state.messageContainerHeight;

      if (_this.refs.chatMessageContainer) {
        var currentHeight = _this.refs.chatMessageContainer.scrollbarRef.getScrollHeight();

        _delay(function () {
          var scrollTopValue = currentHeight - messageContainerHeight;

          _this.refs.chatMessageContainer.scrollbarRef.scrollTop(scrollTopValue);
        }, 100);
      }
    });

    _defineProperty(_assertThisInitialized(_this), "updateMessageContainerHeight", function () {
      _this.setState({
        messageContainerHeight: _this.refs.chatMessageContainer.scrollbarRef.getScrollHeight()
      });
    });

    _this.state = {
      playSoundForNewMessage: false,
      originalMessage: null,
      hasInteracted: false // mouse move, click, tab

    };
    _this.throttledLoadMessageHistory = _throttle(function (channelId) {
      _this.props.actions.getMessages(true, channelId);
    }, 500, {
      trailing: false
    });
    _this.throttledPlaySound = _throttle(function (value) {
      _this.handleSoundStartFinishedPlaying(value);
    }, 2000, {
      trailing: false
    });
    return _this;
  }

  var _proto = ChatMessageContainer.prototype;

  _proto.componentDidMount = function componentDidMount() {
    var _this2 = this;

    //Scroll to bottom when F5 page
    window.addEventListener('click', function () {
      _this2.setState({
        hasInteracted: true
      });
    }, {
      once: true
    });
    window.addEventListener('touchstart', function () {
      _this2.setState({
        hasInteracted: true
      });
    }, {
      once: true
    });
    this.scrollToBottom();
  };

  ChatMessageContainer.getDerivedStateFromProps = function getDerivedStateFromProps(nextProps, prevState) {
    if (nextProps.messages && nextProps.messages !== prevState.originalMessage) {
      return {
        originalMessage: nextProps.messages
      };
    } else {
      return null;
    }
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    var sender = _get(_last(this.state.originalMessage), 'senderId');

    if (prevState.originalMessage !== this.state.originalMessage) {
      if (_size(this.state.originalMessage) - _size(prevState.originalMessage) === 1) {
        if (sender === 'subscriber') {
          this.scrollToBottom();
        } else {
          //playsoud when receiving new message
          if (this.state.hasInteracted && this.props.soundOn && this.props.numberOfNewMessages !== 0) {
            this.throttledPlaySound(true);
          } //is at bottom


          if (this.isScrollingOnBottom()) {
            this.scrollToBottom();
          }
        }
      } //loadmore message


      if (_size(prevState.originalMessage) !== 0 && _size(prevState.originalMessage) < _size(this.state.originalMessage) && this.isScrollingOnTop()) {
        this.scrollToPositionBeforeLoadMore();
      } //update scroll height


      this.updateMessageContainerHeight();
    }
  };

  _proto.render = function render() {
    var _this3 = this;

    var _this$props2 = this.props,
        containerBgColor = _this$props2.containerBgColor,
        isShowLeftAvatar = _this$props2.isShowLeftAvatar,
        isShowRightAvatar = _this$props2.isShowRightAvatar,
        isShowSeenTime = _this$props2.isShowSeenTime,
        avatarWidth = _this$props2.avatarWidth,
        avatarMargin = _this$props2.avatarMargin,
        style = _this$props2.style,
        textFontSize = _this$props2.textFontSize,
        messageMargin = _this$props2.messageMargin,
        onClickButton = _this$props2.onClickButton,
        leftParitcipantsTextColor = _this$props2.leftParitcipantsTextColor,
        leftParitcipantsTextBgColor = _this$props2.leftParitcipantsTextBgColor,
        rightParitcipantsTextColor = _this$props2.rightParitcipantsTextColor,
        rightParitcipantsTextBgColor = _this$props2.rightParitcipantsTextBgColor,
        messages = _this$props2.messages,
        participants = _this$props2.participants;
    return /*#__PURE__*/React.createElement(React.Fragment, null, this.state.playSoundForNewMessage && /*#__PURE__*/React.createElement(Sound, {
      url: sound.SOUND_NEW_MESSAGE,
      playStatus: Sound.status.PLAYING,
      onFinishedPlaying: function onFinishedPlaying() {
        return _this3.handleSoundStartFinishedPlaying(false);
      }
    }), /*#__PURE__*/React.createElement(MessageContainer, {
      containerBgColor: containerBgColor,
      isShowLeftAvatar: isShowLeftAvatar,
      isShowRightAvatar: isShowRightAvatar,
      isShowSeenTime: isShowSeenTime,
      avatarWidth: avatarWidth,
      avatarMargin: avatarMargin,
      style: style,
      textFontSize: textFontSize,
      messageMargin: messageMargin,
      onScrollTop: function onScrollTop() {
        return _this3.loadMoreMessages();
      },
      onScrollBottom: function onScrollBottom() {
        return _this3.scrollToBottom();
      },
      onClickButton: onClickButton,
      leftParitcipantsTextColor: leftParitcipantsTextColor,
      leftParitcipantsTextBgColor: leftParitcipantsTextBgColor,
      rightParitcipantsTextColor: rightParitcipantsTextColor,
      rightParitcipantsTextBgColor: rightParitcipantsTextBgColor,
      messages: messages,
      participants: participants,
      noMessageText: "No messages",
      isChatbox: true,
      ref: "chatMessageContainer"
    }));
  };

  return ChatMessageContainer;
}(React.Component);

export default connect(function (store) {
  return {
    numberOfNewMessages: _get(store, 'messages.numberOfNewMessages')
  };
}, function (dispatch) {
  return {
    actions: bindActionCreators({
      getMessages: getMessages,
      onScrollToBottom: onScrollToBottom
    }, dispatch)
  };
})(ChatMessageContainer);