import _get from "lodash/get";
import _includes from "lodash/includes";
import React, { useContext, Suspense, lazy } from 'react';
import commonStyles from '../../common-styles.scss';
import { getCurrentScreen } from '../../../../commons/utils';
import { useDispatch, useSelector } from 'react-redux';
import { sendButtonMessage } from '../../../../store/actions';
import WidgetContext from '../../../../widget-context';
var UserInfoForm = lazy(function () {
  return import('../../user-info-form');
});
var Header = lazy(function () {
  return import('../../header');
});
var Power = lazy(function () {
  return import('../../power');
});
var ChatBoxInput = lazy(function () {
  return import('../../input');
});
var OfflineStatus = lazy(function () {
  return import('../../offline');
});
var ChatMessageContainer = lazy(function () {
  return import('../message-container');
});
var NoConect = lazy(function () {
  return import('../../no-connection');
});
export default (function (props) {
  var isInConfigurationMode = props.isInConfigurationMode,
      isMobileMode = props.isMobileMode,
      setting = props.setting,
      messages = props.messages,
      participants = props.participants,
      demoData = props.demoData;
  var dispatch = useDispatch();

  var _useContext = useContext(WidgetContext),
      isSimulator = _useContext.isSimulator;

  var showedScreen = isSimulator ? 'conversation' : getCurrentScreen(props);
  var soundOn = isSimulator ? false : useSelector(function (store) {
    return _get(store, 'behavior.soundOn');
  });
  var isViewForOffline = showedScreen === 'off' || showedScreen === 'off_with_chatbot';
  return /*#__PURE__*/React.createElement("div", {
    className: commonStyles.conversationContainer,
    style: {
      padding: isMobileMode ? 0 : "10px"
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: commonStyles.container
  }, isViewForOffline && /*#__PURE__*/React.createElement(Suspense, {
    fallback: /*#__PURE__*/React.createElement("div", null)
  }, /*#__PURE__*/React.createElement(OfflineStatus, {
    setting: setting,
    isInConfigurationMode: isInConfigurationMode,
    showedScreen: showedScreen
  })), !isViewForOffline && /*#__PURE__*/React.createElement(React.Fragment, null, !isSimulator && !_includes(['off', 'no_connection'], showedScreen) && /*#__PURE__*/React.createElement(Suspense, {
    fallback: /*#__PURE__*/React.createElement("div", null)
  }, /*#__PURE__*/React.createElement(Header, {
    setting: setting,
    isInConfigurationMode: isInConfigurationMode,
    demoData: demoData,
    currentScreen: showedScreen
  })), /*#__PURE__*/React.createElement("div", {
    className: commonStyles.boxBody
  }, showedScreen === 'form' && /*#__PURE__*/React.createElement(Suspense, {
    fallback: /*#__PURE__*/React.createElement("div", null)
  }, /*#__PURE__*/React.createElement(UserInfoForm, {
    setting: setting,
    isInConfigurationMode: isInConfigurationMode
  })), showedScreen === 'conversation' && /*#__PURE__*/React.createElement("div", {
    className: commonStyles.messageContainer
  }, /*#__PURE__*/React.createElement(Suspense, {
    fallback: /*#__PURE__*/React.createElement("div", null)
  }, /*#__PURE__*/React.createElement(ChatMessageContainer, {
    containerBgColor: "white",
    isShowLeftAvatar: true,
    isShowRightAvatar: false,
    isShowSeenTime: false,
    avatarWidth: 25,
    avatarMargin: 0,
    style: 1,
    textFontSize: 12,
    messageMargin: 25,
    onScrollTop: function onScrollTop() {},
    onClickButton: function onClickButton(buttonData) {
      return dispatch(sendButtonMessage(buttonData, setting.channelId));
    },
    leftParitcipantsTextColor: _get(setting, 'screenConversation.leftSideTextColor'),
    leftParitcipantsTextBgColor: _get(setting, 'screenConversation.leftSideBackgroundColor'),
    rightParitcipantsTextColor: _get(setting, 'screenConversation.rightSideTextColor'),
    rightParitcipantsTextBgColor: _get(setting, 'screenConversation.rightSideBackgroundColor'),
    messages: isInConfigurationMode ? demoData.messages : messages,
    participants: isInConfigurationMode ? demoData.participants : participants,
    soundOn: soundOn,
    setting: setting
  }))), showedScreen === 'conversation' && /*#__PURE__*/React.createElement(Suspense, {
    fallback: /*#__PURE__*/React.createElement("div", null)
  }, /*#__PURE__*/React.createElement(ChatBoxInput, {
    setting: setting,
    isInConfigurationMode: isInConfigurationMode
  })), showedScreen === 'no_connection' && /*#__PURE__*/React.createElement(Suspense, {
    fallback: /*#__PURE__*/React.createElement("div", null)
  }, /*#__PURE__*/React.createElement(NoConect, {
    setting: setting,
    isInConfigurationMode: isInConfigurationMode,
    demoData: demoData,
    isMobileMode: isMobileMode
  })))), /*#__PURE__*/React.createElement(Power, {
    setting: setting,
    showedScreen: showedScreen
  })));
});