import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _get from "lodash/get";
// https://medium.com/@BoltAssaults/autoplay-muted-html5-video-safari-ios-10-in-react-673ae50ba1f5
import React, { Component } from 'react';
import styles from './styles.scss';
import webRTC from '../../webrtc';
import * as hark from 'hark';
import { CloseCallIcon } from '../../../icon';
import Power from '../power';
import SVG from 'react-inlinesvg';
import endCall from '../../../icon/end-call.svg';
import calling from '../../../icon/calling.svg';
import Avatar from '../../../../commons/components/avatar';
import { CALLING_ICON } from '../../../icon';
import CountTime from '../../../../commons/components/count-time';
import { sendMessage } from '../../../../connections/websocket';
import { GROUP, TYPE } from '../../../../commons/constants/websocket';
import { getTenantInfo } from '../../../../commons/tenant-info';
import { CALL_STOP_REASON } from '../../constants';

var isSafari = function isSafari() {
  var ua = navigator.userAgent.toLowerCase();
  return ua.indexOf('safari') > -1 && ua.indexOf('chrome') < 0;
};

var PageCalling = /*#__PURE__*/function (_Component) {
  _inheritsLoose(PageCalling, _Component);

  function PageCalling(props) {
    var _this;

    _this = _Component.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "state", {
      remoteStream: undefined,
      second: 0,
      minute: 0
    });

    _defineProperty(_assertThisInitialized(_this), "onClickStop", function () {
      var handleStopCall = _this.props.handleStopCall;
      handleStopCall && handleStopCall(CALL_STOP_REASON.SUBSCRIBER_STOP);
      clearInterval(_this.interval);
      clearInterval(_this.intervalSendInCallSignal);
    });

    _defineProperty(_assertThisInitialized(_this), "handleUpdateRemotePeerConnection", function () {
      var callInfo = _this.props.callInfo;

      var remotePeerConnectionInfo = _get(callInfo, 'callData.adminPeerConnectionInfo'); // console.log("configure WebRTC connection", remotePeerConnectionInfo);


      webRTC.updateRemotePeerConnection(remotePeerConnectionInfo);
    });

    _defineProperty(_assertThisInitialized(_this), "handleGotGotRemoteStreamHandler", function () {
      webRTC.onGotRemoteStreamHandler(function (remoteStream) {
        // console.log("webRTC.onGotRemoteStreamHandler: ", remoteStream);
        var adminSpeechEvents = hark(remoteStream, {});
        adminSpeechEvents.on('speaking', function () {
          console.log('AdminTalking');
        });

        _this.setState({
          remoteStream: remoteStream
        }); // adminSpeechEvents.on("stopped_speaking", () => {});
        // const localStream = webRTC.getLocalStream();
        // const localSpeechEvents = hark(localStream, {});
        // localSpeechEvents.on("speaking", () => {
        //   console.log("YouTalking");
        // });
        // localSpeechEvents.on("stopped_speaking", () => {});

      });
    });

    _this.audioParentRef = React.createRef();
    return _this;
  }

  var _proto = PageCalling.prototype;

  _proto.updateRemoteAudioStream = function updateRemoteAudioStream() {
    // console.log("updateRemoteAudioStream");
    var audio = this.audioParentRef.current.children[0];

    if (audio && this.state.remoteStream && audio.srcObject !== this.state.remoteStream) {
      // console.log("setup audio: ", audio);
      if ('srcObject' in audio) {
        // console.log("set audio.srcObject");
        audio.srcObject = this.state.remoteStream;
      } else if ('src' in audio && window.URL.createObjectURL) {
        // console.log("set audio.src");
        audio.src = window.URL.createObjectURL(this.state.remoteStream);
      }

      if (isSafari()) {
        // console.log("isSafari");
        audio.controls = false;
        audio.playsinline = true;
        audio.muted = false; //audio.setAttribute("muted", "");

        audio.autoplay = true;

        if (audio.load) {
          // console.log("audio.load");
          audio.load();
        }

        if (audio.play) {
          console.log('audio.play');
          setTimeout(function () {
            var promise = audio.play();

            if (promise.then) {
              promise.then(function () {// console.log("play successfully");
              })["catch"](function (err) {
                console.log('play failed', err);
              });
            } else {// console.log("is not a promise");
            }
          }, 0);
        }
      } else {
        audio.muted = false;
        audio.controls = false;
        audio.playsinline = true;
        audio.autoplay = true;
        audio.load();
        setTimeout(function () {
          var promise = audio.play();

          if (promise.then) {
            promise.then(function () {// console.log("play successfully");
            })["catch"](function (err) {
              console.log('play failed', err);
            });
          }
        }, 0);
      }

      audio.onloadedmetadata = function () {
        if (isSafari()) {// audio.controls = false;
          // audio.playsinline = true;
          // audio.muted = true;
          // audio.setAttribute("muted", "");
          // audio.autoplay = true;
          // if (audio.play) {
          //   console.log("audio.play");
          //   setTimeout(() => {
          //     const promise = audio.play();
          //     if (promise.then) {
          //       promise
          //         .then(() => {
          //           console.log("play successfully");
          //         })
          //         .catch((err) => {
          //           console.log("play failed", err);
          //         });
          //     }
          //   }, 0);
          // }
        } else {
          setTimeout(function () {
            var promise = audio.play();

            if (promise.then) {
              promise.then(function () {
                console.log('play successfully');
              })["catch"](function (err) {
                console.log('play failed', err);
              });
            }
          }, 0);
        }
      };
    }
  };

  _proto.componentDidMount = function componentDidMount() {
    var _this2 = this;

    this.handleUpdateRemotePeerConnection();
    this.handleGotGotRemoteStreamHandler();
    this.updateRemoteAudioStream(); // Count time

    this.interval = setInterval(function () {
      if (_this2.state.second === 59) {
        _this2.setState({
          second: 0,
          minute: _this2.state.minute + 1
        });
      } else {
        _this2.setState({
          second: _this2.state.second + 1
        });
      }
    }, 1000); // Interval send ttl

    this.intervalSendInCallSignal = setInterval(function () {
      sendMessage({
        group: GROUP.CALL,
        type: TYPE.CALL.UPDATE_IN_CALL_SIGNAL,
        payload: {
          callId: _get(_this2.props.callInfo, ['callData', 'id'])
        },
        tenantInfo: getTenantInfo()
      });
    }, 15000);

    window.parent.onbeforeunload = function () {
      return 'Do not reload page';
    };

    window.parent.addEventListener('unload', this.onClickStop);
  };

  _proto.componentDidUpdate = function componentDidUpdate() {
    this.updateRemoteAudioStream();
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.interval && clearInterval(this.interval);
    this.intervalSendInCallSignal && clearInterval(this.intervalSendInCallSignal);
    window.parent.onbeforeunload = null;
    window.parent.removeEventListener('unload', this.onClickStop);
  };

  _proto.render = function render() {
    var _this$props = this.props,
        setting = _this$props.setting,
        callInfo = _this$props.callInfo,
        isMobileMode = _this$props.isMobileMode;
    var _this$state = this.state,
        minute = _this$state.minute,
        second = _this$state.second;

    var adminInfo = _get(callInfo, ['callData', 'admin']);

    var adminAvatar = _get(adminInfo, 'avatar');

    var adminName = _get(adminInfo, 'displayName') || 'Admin name';

    var isCustomAdminInfo = _get(setting, 'callThemeSettings.isCustomAdminInfo');

    return /*#__PURE__*/React.createElement("div", {
      className: styles.container
    }, /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("div", {
      className: styles.timeContent
    }, /*#__PURE__*/React.createElement("div", {
      className: styles.icon
    }, /*#__PURE__*/React.createElement(SVG, {
      src: calling
    })), /*#__PURE__*/React.createElement(CountTime, {
      minute: minute,
      second: second
    }))), /*#__PURE__*/React.createElement("div", {
      className: styles.infoContent
    }, /*#__PURE__*/React.createElement("div", {
      className: styles.userAvatar
    }, /*#__PURE__*/React.createElement("div", {
      style: {
        backgroundColor: _get(setting, 'screenConversation.rightSideBackgroundColor'),
        color: 'transparent'
      },
      className: styles.wrapOpacity1
    }), /*#__PURE__*/React.createElement("div", {
      style: {
        backgroundColor: _get(setting, 'screenConversation.rightSideBackgroundColor')
      },
      className: styles.wrapOpacity2
    }), !isCustomAdminInfo ? /*#__PURE__*/React.createElement("div", {
      style: {
        backgroundColor: _get(setting, 'screenConversation.rightSideBackgroundColor')
      },
      className: styles.wrap
    }, /*#__PURE__*/React.createElement("div", {
      className: styles.avartar
    }, adminAvatar ? /*#__PURE__*/React.createElement("div", {
      style: {
        width: '60px',
        height: '60px'
      }
    }, /*#__PURE__*/React.createElement(Avatar, {
      size: "60px",
      src: adminAvatar
    })) : /*#__PURE__*/React.createElement(Avatar, {
      size: "30px",
      src: CALLING_ICON.CALLING3
    }))) : /*#__PURE__*/React.createElement("div", {
      style: {
        backgroundColor: _get(setting, 'screenConversation.rightSideBackgroundColor')
      },
      className: styles.wrap
    }, /*#__PURE__*/React.createElement("div", {
      className: styles.avartar
    }, /*#__PURE__*/React.createElement(Avatar, {
      size: "60px",
      src: _get(setting, 'callThemeSettings.informationSettings.callee_avatar')
    })))), /*#__PURE__*/React.createElement("div", {
      style: {
        marginTop: '9px'
      }
    }, /*#__PURE__*/React.createElement("span", {
      className: styles.userName
    }, isCustomAdminInfo ? _get(setting, 'callThemeSettings.informationSettings.callee_name') : adminName))), /*#__PURE__*/React.createElement("div", {
      className: styles.endButton,
      onClick: this.onClickStop
    }, /*#__PURE__*/React.createElement("div", {
      className: styles.iconContainer
    }, /*#__PURE__*/React.createElement(SVG, {
      src: endCall
    }))), /*#__PURE__*/React.createElement("div", {
      className: styles.power
    }, /*#__PURE__*/React.createElement(Power, {
      setting: setting
    })), /*#__PURE__*/React.createElement("div", {
      ref: this.audioParentRef,
      style: {
        display: 'none'
      },
      dangerouslySetInnerHTML: {
        __html: "\n                <video\n                  loop\n                  muted\n                  autoplay\n                  playsinline\n                  preload=\"metadata\"\n                >\n                </video>"
      }
    }), /*#__PURE__*/React.createElement(CloseCallIcon, {
      className: styles.closeIcon
    }));
  };

  return PageCalling;
}(Component);

export { PageCalling as default };