import _extends from "@babel/runtime/helpers/extends";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _find from "lodash/find";
import _get from "lodash/get";
import _includes from "lodash/includes";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import React, { Component, useEffect } from 'react';
import Frame from 'react-frame-component';
import callIcon from '../icon/call-widget.svg';
import { checkMediaDevice, toggleCall, updateMediaPermission, startCall, stopCall, selectDevices } from '../../store/actions';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import PageNoSupport from './components/no-support';
import PageGetPermission from './components/get-permission';
import PageStartCall from './components/start-call';
import PageInitCall from './components/init-call';
import PageCalling from './components/calling';
import PageComplete from './components/complete';
import PageBusy from './components/busy-line';
import { CALL_STATUS, CALL_STOP_REASON } from './constants';
import WidgetIcon from '../../commons/components/widget-icon';
import Layout from './components/layout';
import Tooltip from '../tooltip';

var PhoneCall = /*#__PURE__*/function (_Component) {
  _inheritsLoose(PhoneCall, _Component);

  function PhoneCall(props) {
    var _this;

    _this = _Component.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "handleClickCall", function () {
      _this.props.actions.toggleCall();
    });

    _defineProperty(_assertThisInitialized(_this), "handleGetPermission", function (_ref) {
      var isAllow = _ref.isAllow;

      _this.props.actions.updateMediaPermission(isAllow);

      _this.props.actions.checkMediaDevice();
    });

    _defineProperty(_assertThisInitialized(_this), "handleStartCall", /*#__PURE__*/function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(speakerId, microphoneId) {
        var setting;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                setting = _this.props.setting;

                _this.props.actions.startCall({
                  speakerId: speakerId,
                  microphoneId: microphoneId
                }, setting.channelId);

              case 2:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      return function (_x, _x2) {
        return _ref2.apply(this, arguments);
      };
    }());

    _defineProperty(_assertThisInitialized(_this), "handleSelectDevices", function (speakerId, microphoneId) {
      _this.props.actions.selectDevices({
        speakerId: speakerId,
        microphoneId: microphoneId
      });
    });

    _defineProperty(_assertThisInitialized(_this), "handleStopCall", function (stopReason, totalDuration) {
      if (_this.props.isInConfigurationMode) return;
      var _this$props = _this.props,
          callInfo = _this$props.callInfo,
          setting = _this$props.setting;

      _this.props.actions.stopCall(callInfo.callData, {
        stopReason: stopReason || CALL_STOP_REASON.SUBSCRIBER_STOP,
        totalDuration: totalDuration
      }, setting.channelId);
    });

    _defineProperty(_assertThisInitialized(_this), "calculateWidgetPosition", function () {
      var _this$props2 = _this.props,
          position = _this$props2.position,
          area = _this$props2.area,
          setting = _this$props2.setting,
          showChat = _this$props2.showChat,
          rect = _this$props2.rect,
          isSettingCollapse = _this$props2.isSettingCollapse,
          isSettingLeftBottom = _this$props2.isSettingLeftBottom,
          isSettingVertical = _this$props2.isSettingVertical,
          marginBottom = _this$props2.marginBottom,
          marginLeft = _this$props2.marginLeft,
          marginRight = _this$props2.marginRight;
      var iconSize = setting.iconSize,
          iconMargin = setting.iconMargin;

      var chatboxPosition = _get(setting, 'chatboxComponent.position');

      var leftBottomAlign = _get(setting, 'leftBottomAlign');

      var rightBottomAlign = _get(setting, 'rightBottomAlign');

      var windowWidth = rect.width;
      var bottom = 10;
      var right = 0;
      var width = 400;
      var isChatBoxInSameArea = area === chatboxPosition;
      var isLeftBottomAlignHorizontal = leftBottomAlign === 'horizontal';
      var isRightBottomAlignHorizontal = rightBottomAlign === 'horizontal';

      if (isSettingCollapse) {
        if (isSettingLeftBottom) {
          if (showChat) {
            if (isSettingVertical) {
              right = position.right - width - width - iconMargin - iconMargin + 30;
              bottom = 0;
            } else {
              bottom = 10 + iconSize;
              right = windowWidth - marginRight - width - 10 - width - 10 + 40;
            }
          } else {
            // Don't care the chatbox now in this logic branch
            if (isSettingVertical) {
              right = position.right - width - iconMargin + 10;
              bottom = 0;
            } else {
              bottom = 10 + iconSize;
              right = windowWidth - width - marginLeft + 10;
            }
          }
        } else {
          if (showChat) {
            if (isSettingVertical) {
              right = position.right + 380 + marginRight + iconSize;
              bottom = 0;
            } else {
              bottom = 10 + iconSize;
              right = marginRight + width - 20;
            }
          } else {
            if (isSettingVertical) {
              bottom = 0;
              right = marginRight + iconSize;
            } else {
              bottom = iconSize + 10;
              right = marginRight - 10;
            }
          }
        }
      } else {
        if (isSettingLeftBottom) {
          // If chatbox is in them same area and showing.
          if (showChat) {
            if (isSettingVertical) {
              bottom = marginBottom;
              right = position.right - width - width - iconMargin - iconMargin;
            } else {
              bottom = marginBottom + 10 + iconSize;
              right = windowWidth - marginLeft - 10 - width - width;
            }
          } else {
            // Don't care the chatbox now in this logic branch
            if (isSettingVertical) {
              bottom = marginBottom;
              right = windowWidth - width - 10 - marginLeft - iconSize;
            } else {
              bottom = marginBottom + iconSize + iconMargin;
              right = windowWidth - width - marginLeft;
            }
          }
        } else {
          if (showChat) {
            if (isSettingVertical) {
              bottom = marginBottom;
              right = marginBottom + 10 + +iconSize + width + 10;
            } else {
              right = width + 10 + marginRight;
              bottom = marginBottom + 10 + iconSize;
            }
          } else {
            if (isSettingVertical) {
              right = marginRight + 10 + iconSize;
              bottom = marginBottom;
            } else {
              right = marginRight;
              bottom = marginBottom + 10 + iconSize;
            }
          }
        }
      }

      return {
        bottom: bottom,
        right: right,
        width: width
      };
    });

    _this.state = {
      isHover: false
    };
    _this.iframeRef = React.createRef();
    return _this;
  }

  var _proto = PhoneCall.prototype;

  _proto.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    if (prevProps.iframeStyles !== this.props.iframeStyles) {
      if (this.iframeRef.current) {
        var stylesheetNode = _find(this.iframeRef.current.node.contentDocument.styleSheets, function (node) {
          return _get(node, 'ownerNode.className') === 'iframe-boostcrm';
        });

        if (stylesheetNode && stylesheetNode.ownerNode) {
          stylesheetNode.ownerNode.outerHTML = this.props.iframeStyles;
        }
      }
    }
  };

  _proto.componentDidMount = /*#__PURE__*/function () {
    var _componentDidMount = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              this.props.actions.checkMediaDevice();

            case 1:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function componentDidMount() {
      return _componentDidMount.apply(this, arguments);
    }

    return componentDidMount;
  }();

  _proto.renderCallWidget = function renderCallWidget() {
    var _this2 = this,
        _ref3;

    var _this$props3 = this.props,
        demoData = _this$props3.demoData,
        isInConfigurationMode = _this$props3.isInConfigurationMode,
        isSettingLeftBottom = _this$props3.isSettingLeftBottom;

    var showCall = _get(this.props, 'callInfo.isShowCall');

    var currentComponent = _get(demoData, 'currentComponent');

    var isShowCall = isInConfigurationMode && currentComponent === 'call' || showCall;
    var callInfo = isInConfigurationMode ? demoData.callInfo : this.props.callInfo;
    var isAllowAccessMicrophone = callInfo.isAllowAccessMicrophone,
        isDeviceSupport = callInfo.isDeviceSupport,
        devices = callInfo.devices,
        selectedMicrophoneId = callInfo.selectedMicrophoneId,
        selectedSpeakerId = callInfo.selectedSpeakerId;
    var _this$props4 = this.props,
        isMobileMode = _this$props4.isMobileMode,
        iframeStyles = _this$props4.iframeStyles,
        setting = _this$props4.setting;

    var _this$calculateWidget = this.calculateWidgetPosition(),
        bottom = _this$calculateWidget.bottom,
        right = _this$calculateWidget.right,
        width = _this$calculateWidget.width;

    var status = isInConfigurationMode ? demoData.status : _get(callInfo, 'callData.status');
    var isShowGetPermission = isInConfigurationMode ? status === CALL_STATUS.ACCESS_REQUEST : !isAllowAccessMicrophone;

    var isBorder = isShowGetPermission || _includes([CALL_STATUS.START_CALL, CALL_STATUS.CONNECTING], status);

    var isStartCall = !status || _includes([CALL_STATUS.START_CALL, CALL_STATUS.CONNECTING], status);

    var isLeftSite = _get(setting, 'display.position') === 'LEFT_BOTTOM';
    var backgroundColor = _includes([CALL_STATUS.INITIATED, CALL_STATUS.QUEUED, CALL_STATUS.RINGING, CALL_STATUS.IN_PROGRESS], status) ? _get(setting, 'iconBackgroundColor') || "#fff" : '#fff';

    var renderWhenDeviceSupports = function renderWhenDeviceSupports() {
      return /*#__PURE__*/React.createElement(Layout, {
        isShowCall: isShowCall,
        setting: setting,
        isMobileMode: isMobileMode,
        border: isBorder ? true : false,
        backgroundColor: backgroundColor,
        isLeftSite: isLeftSite
      }, isShowGetPermission && /*#__PURE__*/React.createElement(PageGetPermission, {
        handleGetPermission: _this2.handleGetPermission.bind(_this2),
        setting: setting,
        isMobileMode: isMobileMode
      }), isStartCall && /*#__PURE__*/React.createElement(PageStartCall, {
        callInfo: callInfo,
        devices: devices,
        handleStartCall: _this2.handleStartCall.bind(_this2),
        handleSelectDevices: _this2.handleSelectDevices.bind(_this2),
        selectedMicrophoneId: selectedMicrophoneId,
        selectedSpeakerId: selectedSpeakerId,
        setting: setting,
        isMobileMode: isMobileMode
      }), _includes([CALL_STATUS.INITIATED, CALL_STATUS.QUEUED, CALL_STATUS.RINGING], status) && /*#__PURE__*/React.createElement(PageInitCall, {
        callInfo: callInfo,
        handleStopCall: _this2.handleStopCall.bind(_this2),
        setting: setting,
        isMobileMode: isMobileMode
      }), status === CALL_STATUS.IN_PROGRESS && /*#__PURE__*/React.createElement(PageCalling, {
        callInfo: callInfo,
        handleStopCall: _this2.handleStopCall.bind(_this2),
        setting: setting,
        isMobileMode: isMobileMode
      }), _includes([CALL_STATUS.BUSY, CALL_STATUS.ADMIN_CANCELED, CALL_STATUS.CUSTOMER_CANCELED, CALL_STATUS.NO_ANSWER, CALL_STATUS.BUSY_LEAVE_INFO, CALL_STATUS.FAILED], status) && /*#__PURE__*/React.createElement(PageBusy, {
        status: status,
        setting: setting,
        handleCallAgain: _this2.handleStartCall.bind(_this2),
        isMobileMode: isMobileMode
      }), _includes([CALL_STATUS.COMPLETED, CALL_STATUS.COMPLETED_RATING], status) && /*#__PURE__*/React.createElement(PageComplete, {
        status: status,
        setting: setting,
        callInfo: callInfo,
        handleMakeAnotherCall: _this2.handleStartCall.bind(_this2),
        isMobileMode: isMobileMode
      }), status === CALL_STATUS.NO_SUPPORT && /*#__PURE__*/React.createElement(PageNoSupport, {
        status: status,
        setting: setting,
        isMobileMode: isMobileMode
      }));
    };

    return /*#__PURE__*/React.createElement(Frame, {
      ref: this.iframeRef,
      initialContent: "<!DOCTYPE html><html><head>          \n          <link href=\"https://fonts.googleapis.com/css?family=Comfortaa&display=swap\" rel=\"stylesheet\">\n          <link href=\"https://fonts.googleapis.com/css?family=Roboto&display=swap\" rel=\"stylesheet\">\n          <link href=\"https://fonts.googleapis.com/css2?family=Open+Sans&display=swap\" rel=\"stylesheet\">\n          <style>\n            .bs-chatbox-input-container .ant-upload {\n              display: flex !important;\n            }\n            .bs-call-form .ant-form-item-control {\n              line-height: 31px !important;\n            }\n          </style>\n          " + iframeStyles + "</head><body></body></html>\n        ",
      mountTarget: "body",
      allowtransparency: "yes",
      style: (_ref3 = {
        right: isMobileMode ? 0 : right + "px",
        bottom: isMobileMode ? 0 : bottom + "px",
        position: isMobileMode ? 'fixed' : 'absolute',
        width: isMobileMode ? '100%' : width + "px",
        minHeight: '468px',
        maxHeight: isMobileMode ? '100%' : '468px',
        opacity: isShowCall ? '1' : '0',
        transform: isShowCall ? 'scale(1, 1)' : 'scale(0, 0)',
        height: isMobileMode ? '100%' : "calc(100% - " + bottom + "px)",
        zIndex: 9999999,
        transition: "opacity 0.8s",
        overflow: 'hidden',
        transformOrigin: isSettingLeftBottom ? "bottom left" : "bottom right"
      }, _ref3["transition"] = "transform 0.3s ease-in-out,opacity 0.3s ease-in-out , box-shadow 0.3s ease-in-out , right 0.3s ease-in-out", _ref3.maxWidth = 'unset', _ref3),
      frameBorder: "0"
    }, /*#__PURE__*/React.createElement("div", {
      style: {
        width: '100%',
        height: '100%'
      }
    }, !isDeviceSupport && /*#__PURE__*/React.createElement(Layout, {
      setting: setting,
      isMobileMode: isMobileMode,
      backgroundColor: backgroundColor
    }, /*#__PURE__*/React.createElement(PageNoSupport, {
      status: status,
      setting: setting,
      isMobileMode: isMobileMode
    })), isDeviceSupport && renderWhenDeviceSupports()));
  };

  _proto.render = function render() {
    var _this3 = this;

    var _this$props5 = this.props,
        iframeStyles = _this$props5.iframeStyles,
        position = _this$props5.position,
        setting = _this$props5.setting,
        svgSize = _this$props5.svgSize;
    var iconBackgroundColor = setting.iconBackgroundColor,
        iconTextColor = setting.iconTextColor;

    var iconSize = _get(setting, 'iconSize');

    return /*#__PURE__*/React.createElement("div", null, this.renderCallWidget(), /*#__PURE__*/React.createElement(Frame, {
      initialContent: "<!DOCTYPE html><html><head>" + iframeStyles + "</head><body></body></html>",
      mountTarget: "body",
      style: {
        width: iconSize,
        height: iconSize,
        position: 'absolute',
        bottom: position.bottom + "px",
        right: position.right + "px",
        transition: "opacity 0.8s",
        zIndex: '9999997',
        maxWidth: iconSize,
        minWidth: iconSize,
        minHeight: iconSize,
        maxHeight: iconSize
      },
      allowtransparency: "yes",
      frameBorder: "0"
    }, /*#__PURE__*/React.createElement(WidgetIcon, {
      iconBackgroundColor: iconBackgroundColor,
      iconSize: iconSize,
      iconSrc: callIcon,
      iconTextColor: iconTextColor,
      svgSize: svgSize,
      onClick: this.handleClickCall,
      handleHover: function handleHover(e) {
        return _this3.setState(e);
      }
    })), this.state.isHover && /*#__PURE__*/React.createElement(Tooltip, _extends({}, this.props, this.state)));
  };

  return PhoneCall;
}(Component);

export default connect(function (store) {
  return {
    callInfo: _get(store, 'behavior.callInfo'),
    showChat: _get(store, 'behavior.showChat')
  };
}, function (dispatch) {
  return {
    actions: bindActionCreators({
      checkMediaDevice: checkMediaDevice,
      toggleCall: toggleCall,
      updateMediaPermission: updateMediaPermission,
      startCall: startCall,
      stopCall: stopCall,
      selectDevices: selectDevices
    }, dispatch)
  };
})(PhoneCall);