"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.NetworkDetector = void 0;

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _inheritsLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/inheritsLoose"));

var _react = require("react");

var _propTypes = _interopRequireDefault(require("prop-types"));

// Custom from lib react-detect-offline
var inBrowser = typeof navigator !== 'undefined'; // these browsers don't fully support navigator.onLine, so we need to use a polling backup

var unsupportedUserAgentsPattern = /Windows.*Chrome|Windows.*Firefox|Linux.*Chrome/;

var ping = function ping(_ref) {
  var url = _ref.url,
      timeout = _ref.timeout;
  return new Promise(function (resolve) {
    var isOnline = function isOnline() {
      return resolve(true);
    };

    var isOffline = function isOffline() {
      return resolve(false);
    };

    var xhr = new XMLHttpRequest();
    xhr.onerror = isOffline;
    xhr.ontimeout = isOffline;

    xhr.onreadystatechange = function () {
      if (xhr.readyState === xhr.HEADERS_RECEIVED) {
        if (xhr.status) {
          isOnline();
        } else {
          isOffline();
        }
      }
    };

    xhr.open('HEAD', url);
    xhr.timeout = timeout;
    xhr.send();
  });
};

var propTypes = process.env.NODE_ENV !== "production" ? {
  children: _propTypes["default"].node,
  onChange: _propTypes["default"].func,
  polling: _propTypes["default"].oneOfType([_propTypes["default"].shape({
    url: _propTypes["default"].string,
    interval: _propTypes["default"].number,
    timeout: _propTypes["default"].number
  }), _propTypes["default"].bool]),
  wrapperType: _propTypes["default"].string
} : {};
var defaultProps = {
  polling: true,
  wrapperType: 'span'
};
var defaultPollingConfig = {
  enabled: inBrowser && unsupportedUserAgentsPattern.test(navigator.userAgent),
  url: 'https://api.boostcrm.io/',
  timeout: 5000,
  interval: 5000
}; // base class that detects offline/online changes

var Base = /*#__PURE__*/function (_PureComponent) {
  (0, _inheritsLoose2["default"])(Base, _PureComponent);

  function Base() {
    var _this;

    _this = _PureComponent.call(this) || this;
    _this.state = {
      online: inBrowser && typeof navigator.onLine === 'boolean' ? navigator.onLine : true
    }; // bind event handlers

    _this.goOnline = _this.goOnline.bind((0, _assertThisInitialized2["default"])(_this));
    _this.goOffline = _this.goOffline.bind((0, _assertThisInitialized2["default"])(_this));
    return _this;
  }

  var _proto = Base.prototype;

  _proto.componentDidMount = function componentDidMount() {
    window.addEventListener('online', this.goOnline);
    window.addEventListener('offline', this.goOffline);

    if (this.getPollingConfig().enabled) {
      this.startPolling();
    }
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    window.removeEventListener('online', this.goOnline);
    window.removeEventListener('offline', this.goOffline);

    if (this.pollingId) {
      this.stopPolling();
    }
  };

  _proto.renderChildren = function renderChildren() {
    var _this$props = this.props,
        children = _this$props.children,
        wrapperType = _this$props.wrapperType; // usual case: one child that is a react Element

    if ((0, _react.isValidElement)(children)) {
      return children;
    } // no children


    if (!children) {
      return null;
    } // string children, multiple children, or something else


    return _react.createElement.apply(void 0, [wrapperType, {}].concat(_react.Children.toArray(children)));
  };

  _proto.getPollingConfig = function getPollingConfig() {
    switch (this.props.polling) {
      case true:
        return defaultPollingConfig;

      case false:
        return {
          enabled: false
        };

      default:
        return Object.assign({}, defaultPollingConfig, this.props.polling);
    }
  };

  _proto.goOnline = function goOnline() {
    if (!this.state.online) {
      this.callOnChangeHandler(true);
      this.setState({
        online: true
      });
    }
  };

  _proto.goOffline = function goOffline() {
    if (this.state.online) {
      this.callOnChangeHandler(false);
      this.setState({
        online: false
      });
    }
  };

  _proto.callOnChangeHandler = function callOnChangeHandler(online) {
    if (this.props.onChange) {
      this.props.onChange(online);
    }
  };

  _proto.startPolling = function startPolling() {
    var _this2 = this;

    var _this$getPollingConfi = this.getPollingConfig(),
        interval = _this$getPollingConfi.interval;

    this.pollingId = setInterval(function () {
      var _this2$getPollingConf = _this2.getPollingConfig(),
          url = _this2$getPollingConf.url,
          timeout = _this2$getPollingConf.timeout;

      ping({
        url: url,
        timeout: timeout
      }).then(function (online) {
        online ? _this2.goOnline() : _this2.goOffline();
      });
    }, interval);
  };

  _proto.stopPolling = function stopPolling() {
    clearInterval(this.pollingId);
  };

  _proto.render = function render() {
    return null;
  };

  return Base;
}(_react.PureComponent);

Base.propTypes = process.env.NODE_ENV !== "production" ? propTypes : {};
Base.defaultProps = defaultProps;

var NetworkDetector = /*#__PURE__*/function (_Base) {
  (0, _inheritsLoose2["default"])(NetworkDetector, _Base);

  function NetworkDetector() {
    return _Base.apply(this, arguments) || this;
  }

  var _proto2 = NetworkDetector.prototype;

  _proto2.render = function render() {
    return null;
  };

  return NetworkDetector;
}(Base);

exports.NetworkDetector = NetworkDetector;
NetworkDetector.propTypes = process.env.NODE_ENV !== "production" ? propTypes : {};
NetworkDetector.defaultProps = defaultProps;