"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _findLastIndex2 = _interopRequireDefault(require("lodash/findLastIndex"));

var _size2 = _interopRequireDefault(require("lodash/size"));

var _unionBy2 = _interopRequireDefault(require("lodash/unionBy"));

var _find2 = _interopRequireDefault(require("lodash/find"));

var _get2 = _interopRequireDefault(require("lodash/get"));

var _pick2 = _interopRequireDefault(require("lodash/pick"));

var _isEmpty2 = _interopRequireDefault(require("lodash/isEmpty"));

var _store = require("../../commons/utils/store");

var _correctMessage = require("../../commons/utils/correct-message");

var _messageStatus = require("../../commons/constants/message-status");

var _senderTypes = require("../../commons/constants/sender-types");

var _actions = require("../actions");

var _messageReducer;

function _createForOfIteratorHelperLoose(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (it) return (it = it.call(o)).next.bind(it); if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; return function () { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var MAX_NUMBER_OF_MESSAGES = 20;
var defaultIdForSubscriber = 'subscriber';
var defaultIdForBot = 'bot';

var getSenderId = function getSenderId(message) {
  if (!message) return;

  if (message.senderType === _senderTypes.SENDER_TYPES.BOT) {
    return defaultIdForBot;
  }

  if (message.senderType === _senderTypes.SENDER_TYPES.SUBSCRIBER) {
    return defaultIdForSubscriber;
  }

  return message.adminId || defaultIdForBot;
};

function getMessageFromPayload(payload) {
  var type = (0, _get2["default"])(payload, 'type');

  if (type === 'text') {
    return (0, _extends2["default"])({}, (0, _pick2["default"])(payload, ['id', 'type', 'status', 'externalId']), {
      content: (0, _get2["default"])(payload, 'content.text') || (0, _get2["default"])(payload, 'content')
    });
  }

  return (0, _pick2["default"])(payload, ['id', 'type', 'status', 'content', 'externalId']);
}

function shortifyMessages(messages) {
  // return messages;
  if (!messages) return [];

  if ((0, _size2["default"])(messages) > MAX_NUMBER_OF_MESSAGES) {
    // console.log('messages before: ', size(messages));
    var results = messages.slice((0, _size2["default"])(messages) - MAX_NUMBER_OF_MESSAGES); // console.log('messages after: ', size(results));

    return results;
  }

  return messages;
}

function mergeOrAppendNewMessages(oldMessages, newMessages) {
  var result = (0, _unionBy2["default"])(oldMessages, newMessages, function (m) {
    return m.message.id;
  }).sort(function (a, b) {
    return a.message.id - b.message.id;
  }); // console.log('mergeOrAppendNewMessages: ');
  // console.log('newMessages: ', JSON.stringify(newMessages));
  // console.log('oldMessages: ', JSON.stringify(oldMessages));
  // Merge new fields to the existing one

  var _loop = function _loop() {
    var _step$value = _step.value,
        i = _step$value[0],
        item = _step$value[1];

    if (item.message.id) {
      var foundItem = (0, _find2["default"])(newMessages, function (m) {
        return m.message.id === item.message.id;
      });

      if (foundItem) {
        // console.log('found item: ', item.message.id);
        result[i] = (0, _extends2["default"])({}, item, {
          senderType: (0, _get2["default"])(foundItem, 'senderType') || (0, _get2["default"])(item, 'senderType'),
          // For hot fix
          message: (0, _extends2["default"])({}, item.message, foundItem.message)
        });
      } else {// console.log('not found: ', item.message.id);
      }
    }
  };

  for (var _iterator = _createForOfIteratorHelperLoose(result.entries()), _step; !(_step = _iterator()).done;) {
    _loop();
  }

  return (0, _correctMessage.handleCorrectMessages)(result);
}

function getNumberOfNewMessages(messages) {
  var numberOfNewMessages = 0;
  var lastMessageRead = (0, _findLastIndex2["default"])(messages, ['status', _messageStatus.MESSAGE_STATUS.READ]);
  (messages || []).forEach(function (m, index) {
    if (index > lastMessageRead && m.senderId !== 'subscriber' && (0, _get2["default"])(m, ['message', 'type']) !== 'delay') {
      numberOfNewMessages = numberOfNewMessages + 1;
    }
  });
  return numberOfNewMessages;
}

var initialState = {
  rawMessages: [],
  messages: [],
  participants: [{
    id: defaultIdForBot,
    name: 'bot',
    avatar: '',
    side: 'left',
    isShowAvatar: true
  }, {
    id: defaultIdForSubscriber,
    name: 'user01',
    avatar: '',
    side: 'right',
    isShowAvatar: false
  }],
  isLoadingMessage: false,
  isLoadingMessageHistory: false,
  numberOfNewMessages: 0
};
var messageReducer = (_messageReducer = {}, _messageReducer[_actions.onScrollToBottom] = function (state) {
  // console.log('onScrollToBottom');
  if (state.isLoadingMessageHistory) {
    return state;
  }

  return (0, _extends2["default"])({}, state, {
    messages: shortifyMessages(state.messages)
  });
}, _messageReducer[_actions.toggleChatRequest] = function (state, _ref) {
  var payload = _ref.payload;
  return (0, _extends2["default"])({}, state, {
    numberOfNewMessages: payload === true ? 0 : getNumberOfNewMessages(state.messages)
  });
}, _messageReducer[_actions.sendTextMessageRequest] = function (state, _ref2) {
  var payload = _ref2.payload;
  var newMessages = [].concat(state.messages, [{
    senderId: defaultIdForSubscriber,
    senderType: _senderTypes.SENDER_TYPES.SUBSCRIBER,
    message: getMessageFromPayload(payload)
  }]);
  return (0, _extends2["default"])({}, state, {
    messages: newMessages,
    numberOfNewMessages: getNumberOfNewMessages(newMessages)
  });
}, _messageReducer[_actions.receiveCreatedMessageNotification] = function (state, _ref3) {
  var payload = _ref3.payload;
  var id = payload.id,
      status = payload.status,
      message = payload.message;
  return (0, _extends2["default"])({}, state, {
    messages: state.messages.map(function (item) {
      if (item.message.id) return item;

      if ((0, _get2["default"])(item, 'message.externalId') === (0, _get2["default"])(message, 'externalId')) {
        return (0, _extends2["default"])({}, item, {
          message: getMessageFromPayload((0, _extends2["default"])({
            id: id,
            status: status
          }, message))
        });
      }

      return item;
    })
  });
}, _messageReducer[_actions.receiveMessageFromAdmin] = function (state, _ref4) {
  var payload = _ref4.payload;
  var messages = mergeOrAppendNewMessages(state.messages, [{
    senderId: defaultIdForBot,
    senderType: (0, _get2["default"])(payload, 'sender.isBot') ? _senderTypes.SENDER_TYPES.BOT : _senderTypes.SENDER_TYPES.ADMIN,
    message: getMessageFromPayload(payload)
  }]);
  return (0, _extends2["default"])({}, state, {
    messages: messages,
    // messages: shortifyMessages(messages),
    numberOfNewMessages: getNumberOfNewMessages(messages)
  });
}, _messageReducer[_actions.sendAttachmentRequest] = function (state, _ref5) {
  var payload = _ref5.payload;
  return (0, _extends2["default"])({}, state, {
    messages: [].concat(state.messages, [{
      senderId: defaultIdForSubscriber,
      senderType: _senderTypes.SENDER_TYPES.SUBSCRIBER,
      message: getMessageFromPayload(payload)
    }])
  });
}, _messageReducer[_actions.sendAttachmentSuccess] = function (state, action) {
  // Old messages
  var listMessages = [].concat(state.messages); // New messages

  var newMessages = (0, _get2["default"])(action, ['payload', 'res']); // Append new messages

  (newMessages || []).map(function (newMessage) {
    var externalId = (0, _get2["default"])(newMessage, 'externalId');
    listMessages = listMessages.map(function (mes) {
      var originalExternalId = (0, _get2["default"])(mes, ['message', 'externalId']);

      if (Number(originalExternalId) === Number(externalId)) {
        return {
          senderId: defaultIdForSubscriber,
          senderType: _senderTypes.SENDER_TYPES.SUBSCRIBER,
          message: getMessageFromPayload((0, _get2["default"])(newMessage, 'message'))
        };
      }

      return mes;
    });
  });
  return (0, _extends2["default"])({}, state, {
    messages: listMessages
  });
}, _messageReducer[_actions.sendAttachmentError] = function (state, action) {
  return (0, _extends2["default"])({}, state);
}, _messageReducer[_actions.getMessagesRequest] = function (state, _ref6) {
  var payload = _ref6.payload;
  return (0, _extends2["default"])({}, state, {
    isLoadingMessage: true,
    isLoadingMessageHistory: payload.isLoadingMessageHistory
  });
}, _messageReducer[_actions.getMessagesSuccess] = function (state, _ref7) {
  var payload = _ref7.payload;
  var messages = payload || [];
  if ((0, _isEmpty2["default"])(messages)) return (0, _extends2["default"])({}, state, {
    isLoadingMessage: true,
    isLoadingMessageHistory: false,
    numberOfNewMessages: getNumberOfNewMessages(state.message)
  });
  var newMessages = mergeOrAppendNewMessages(state.messages, messages.map(function (m) {
    return {
      senderId: getSenderId(m),
      senderType: m.senderType,
      message: getMessageFromPayload((0, _extends2["default"])({
        id: m.id,
        status: m.status
      }, m.message))
    };
  }));
  return (0, _extends2["default"])({}, state, {
    isLoadingMessage: true,
    isLoadingMessageHistory: false,
    messages: newMessages,
    numberOfNewMessages: getNumberOfNewMessages(newMessages)
  });
}, _messageReducer[_actions.resetNumberOfNewMessages] = function (state) {
  var lastMessage = state.messages.pop();
  return (0, _extends2["default"])({}, state, {
    messages: [].concat(state.messages, [(0, _extends2["default"])({}, lastMessage, {
      status: _messageStatus.MESSAGE_STATUS.READ
    })]),
    numberOfNewMessages: 0
  });
}, _messageReducer);

var _default = function _default(state, action) {
  if (state === void 0) {
    state = initialState;
  }

  return (0, _store.createReducer)(messageReducer, state, action);
};

exports["default"] = _default;
module.exports = exports.default;